import itertools
import sys
from pathlib import Path

from django.apps import apps
from django.conf import settings

from livesync.asyncserver import dispatcher
from livesync.core.event import ClientEvent
from livesync.core.handler import BaseEventHandler
from livesync.core.signals import livesync_event


class DjangoBrythonEventHandler(BaseEventHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.static_dirs = set()
        self.template_dirs = set()

    @property
    def watched_paths(self):
        paths = {Path(settings.BASE_DIR)}
        for app in apps.get_app_configs():
            paths.add(Path(app.path))
            if tmpl_dir := (Path(app.path) / 'template').exists():
                self.template_dirs.add(tmpl_dir)

        #for path in getattr(settings, 'STATICFILES_DIRS', []):
        #    paths.add(Path(path))

        paths = set(itertools.chain(
            getattr(settings, 'STATICFILES_DIRS', []),
            paths,
        ))

        paths.update(*[tmpl['DIRS'] for tmpl in settings.TEMPLATES])
        return paths

    def is_ignored_pattern(self, path):
        path = Path(path)
        if path.suffix.endswith('~'):  # temporary file
            return True
        elif '.pyc' in path.suffixes:  # compiled python file
            return True

        return False

    def on_modified(self, event):
        if self.is_ignored_pattern(event.src_path):
            return

        super().on_modified(event)

    def on_moved(self, event):
        if self.is_ignored_pattern(event.src_path):
            return

        super().on_moved(event)

    def handle(self, event):
        module_path = event.src_path
        livesync_event.send(sender=self.__class__, event=ClientEvent(
            action='refresh',
            parameters={}
        ))
        return
        if module_path.endswith('.py'):
            for path in sys.path:
                if module_path.startswith(str(path)):
                    # /sys/path/module/path.py -> module.path
                    module_name = module_path[len(path)+1:-3].replace('/', '.')
                    livesync_event.send(sender=self.__class__, event=ClientEvent(
                        action='reload_brython_module',
                        parameters={
                            'module_name': module_name
                        }
                    ))



import posixpath

from importlib.util import find_spec
from pathlib import Path

from django.conf import settings

from django.http import Http404, HttpResponse
from django.views import static


def serve(request, path, **kwargs):
    if settings.DEBUG is False and getattr(settings, 'BRYTHON_UNITTEST_RUNNING', False) is False:
        raise Http404(f'To access brython files from server please set DEBUG or BRYTHON_UNITTEST_RUNNING to True')

    module_path = posixpath.normpath(path).lstrip('/').replace('.py', '').replace('/', '.')

    if module_path == 'django_brython.testing':
        module_path = 'django_brython.mock_testing'

    try:
        module = find_spec(module_path)
    except ModuleNotFoundError:
        raise Http404(f"'import {module_path}' exception, file: {path} can't be served")

    if not module:
        raise Http404(f"'import {module_path}' exception, file: {path} can't be served")

    tested_module_path = Path(module.origin)

    if not str(tested_module_path).endswith(path) and module_path != 'django_brython.mock_testing':
        raise Http404(f"Requested and imported module doesn't match. Imported: {module.origin}, requested: {path}")

    return static.serve(request, tested_module_path.name, document_root=tested_module_path.parent, **kwargs)


def require(request):
    module_path = posixpath.normpath(request.GET.get('caller')).lstrip('/').replace('.py', '').replace('/', '.').replace('VFS.', '')
    required = request.GET.get('required').split('?')[0]

    try:
        module = find_spec(module_path)
    except ModuleNotFoundError:
        raise Http404(f"'import {module_path}' exception, file: {required} can't be served")

    if not module:
        raise Http404(f"'import {module_path}' exception, file: {required} can't be served")

    path = Path(module.origin).parent / required

    resp = static.serve(request, path.name, document_root=path.parent)
    resp['Content-Type'] = 'text/plain'
    return resp



import sys
import traceback

from browser import timer, window


def location(url, driver='driver', wait_for=None, function=None, instance=None):
    if function is None and instance is None:
        def dec(func):
            return location(url, driver=driver, wait_for=wait_for, function=func)

        return dec

    if function is not None and instance is None:
        def dec(inst):
            if wait_for:
                def wait_for_condition():
                    result = wait_for()
                    if result is True:
                        timer.clear_interval(test_timer)
                        try:
                            return function(inst)
                        except:
                            exc = traceback.format_exc()
                            window.__BRYTHON_UNITTEST_RESULTS__ = {
                                'exception': exc
                            }
                    else:
                        print("'wait_for' condition is not fulfilled, wait for one second")

                test_timer = timer.set_interval(wait_for_condition, 1000)

            else:
                try:
                    return function(inst)
                except:
                    exc = traceback.format_exc()
                    print(exc, file=sys.stderr)
                    window.__BRYTHON_UNITTEST_RESULTS__ = {
                        'exception': exc
                    }

        return dec


def reverse(*args, **kwargs):
    pass


def reverse_lazy(*args, **kwargs):
    pass
